@@ -9,7 +9,7 @@ from waiter import http_util, terminal
 from waiter.format import format_last_request_time
 from waiter.format import format_status
 from waiter.querying import get_service, get_services_using_token
-from waiter.querying import print_no_data, query_service, query_services
+from waiter.querying import print_no_data, query_service, query_services, query_token
 from waiter.util import is_service_current, str2bool, response_message, print_error, wait_until
 
 
@@ -237,3 +237,48 @@ def process_kill_request(clusters, token_name_or_service_id, is_service_id, forc
                 success = kill_service_on_cluster(cluster, service_id, timeout_secs)
                 overall_success = overall_success and success
     return overall_success
+
+
+def token_explicitly_created_on_cluster(cluster, token_cluster_name):
+    """Returns true if the given token cluster matches the configured cluster name of the given cluster"""
+    cluster_settings, _ = http_util.make_data_request(cluster, lambda: http_util.get(cluster, '/settings'))
+    cluster_config_name = cluster_settings['cluster-config']['name'].upper()
+    created_on_this_cluster = token_cluster_name == cluster_config_name
+    return created_on_this_cluster
+
+
+def process_ping_request(clusters, token_name_or_service_id, is_service_id, timeout_secs, wait_for_request):
+    """Pings the service using the given token name or service-id.
+    Returns False if the ping request fails or times out.
+    Returns True if the ping request completes successfully."""
+    if is_service_id:
+        query_result = query_service(clusters, token_name_or_service_id)
+    else:
+        query_result = query_token(clusters, token_name_or_service_id)
+
+    if query_result['count'] == 0:
+        print_no_data(clusters)
+        return False
+
+    http_util.set_retries(0)
+    cluster_data_pairs = sorted(query_result['clusters'].items())
+    clusters_by_name = {c['name']: c for c in clusters}
+    overall_success = True
+    for cluster_name, data in cluster_data_pairs:
+        cluster = clusters_by_name[cluster_name]
+        if is_service_id:
+            success = ping_service_on_cluster(cluster, token_name_or_service_id, timeout_secs, wait_for_request)
+        else:
+            token_data = data['token']
+            token_etag = data['etag']
+            token_cluster_name = token_data['cluster'].upper()
+            if len(clusters) == 1 or token_explicitly_created_on_cluster(cluster, token_cluster_name):
+                success = ping_token_on_cluster(cluster, token_name_or_service_id, timeout_secs,
+                                                wait_for_request, token_etag)
+            else:
+                print(f'Not pinging token {terminal.bold(token_name_or_service_id)} '
+                      f'in {terminal.bold(cluster_name)} '
+                      f'because it was created in {terminal.bold(token_cluster_name)}.')
+                success = True
+        overall_success = overall_success and success
+    return overall_success
